@@ -10,11 +10,11 @@ sys.path.insert(0, path)
 # Not correct, but enough to get the tests going uninstalled
 __builtin__.__dict__['DATADIR'] = path
 
-from giscanner.annotationparser import AnnotationParser
+from giscanner.annotationparser import GtkDocCommentBlockParser
 from giscanner.ast import Include, Namespace
 from giscanner.introspectablepass import IntrospectablePass
 from giscanner.maintransformer import MainTransformer
-from giscanner.message import MessageLogger
+from giscanner.message import MessageLogger, WARNING, ERROR, FATAL
 from giscanner.sourcescanner import SourceScanner
 from giscanner.transformer import Transformer
 from giscanner.scannermain import process_packages
@@ -60,21 +60,18 @@ def _diff(a, b):
             if tag == 'equal':
                 for line in a[i1:i2]:
                     for l in line.split('\n'):
-                        if l != '':
-                            retval += ' ' + l + '\n'
+                        retval += ' ' + l + '\n'
                 continue
 
             if tag in ('replace', 'delete'):
                 for line in a[i1:i2]:
                     for l in line.split('\n'):
-                        if l != '':
-                            retval += '-' + l + '\n'
+                        retval += '-' + l + '\n'
 
             if tag in ('replace', 'insert'):
                 for line in b[j1:j2]:
                     for l in line.split('\n'):
-                        if l != '':
-                            retval += '+' + l + '\n'
+                        retval += '+' + l + '\n'
 
     return retval
 
@@ -100,7 +97,7 @@ def check(args):
     output = ChunkedIO()
     namespace = Namespace('Test', '1.0')
     logger = MessageLogger.get(namespace=namespace, output=output)
-    logger.enable_warnings(True)
+    logger.enable_warnings((WARNING, ERROR, FATAL))
 
     transformer = Transformer(namespace)
     transformer.set_include_paths([os.path.join(top_srcdir, 'gir'), top_builddir])
@@ -117,8 +114,8 @@ def check(args):
     ss.parse_macros([filename])
     transformer.parse(ss.get_symbols())
 
-    ap = AnnotationParser()
-    blocks = ap.parse(ss.get_comments())
+    cbp = GtkDocCommentBlockParser()
+    blocks = cbp.parse_comment_blocks(ss.get_comments())
 
     main = MainTransformer(transformer, blocks)
     main.transform()
@@ -130,8 +127,9 @@ def check(args):
 
     expected_warnings = _extract_expected(filename)
 
-    expected_warnings.sort()
-    emitted_warnings.sort()
+    sortkey = lambda x: int(x.split(':')[0])
+    expected_warnings.sort(key=sortkey)
+    emitted_warnings.sort(key=sortkey)
 
     if len(expected_warnings) != len(emitted_warnings):
         raise SystemExit('ERROR in %r: %d warnings were emitted, '
