@@ -36,7 +36,6 @@
 #include "girparser.h"
 #include "gitypelib-internal.h"
 
-gboolean no_init = FALSE;
 gchar **includedirs = NULL;
 gchar **input = NULL;
 gchar *output = NULL;
@@ -129,7 +128,6 @@ static void log_handler (const gchar *log_domain,
 
 static GOptionEntry options[] = 
 {
-  { "no-init", 0, 0, G_OPTION_ARG_NONE, &no_init, "do not create _init() function", NULL },
   { "includedir", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &includedirs, "include directories in GIR search path", NULL }, 
   { "output", 'o', 0, G_OPTION_ARG_FILENAME, &output, "output file", "FILE" }, 
   { "module", 'm', 0, G_OPTION_ARG_STRING, &mname, "module to compile", "NAME" }, 
@@ -155,6 +153,15 @@ main (int argc, char ** argv)
   g_option_context_parse (context, &argc, &argv, &error);
   g_option_context_free (context);
 
+  if (error)
+    {
+      g_fprintf (stderr, "error parsing arguments: %s\n", error->message);
+
+      g_error_free (error);
+
+      return 1;
+    }
+
   logged_levels = G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_MESSAGE|G_LOG_LEVEL_DEBUG);
   if (debug)
     logged_levels = logged_levels | G_LOG_LEVEL_DEBUG;
